@@ -46,6 +46,7 @@ import hashlib
 import zipfile
 import tempfile
 from getpass import getpass
+from contextlib import contextmanager
 
 import requests
 from tqdm import tqdm
@@ -137,6 +138,37 @@ def _cache_response(method):
     return wrapper_decorator
 
 
+@contextmanager
+def no_cache(ac=None):
+    """Temporarily turn off the REST cache for a given Alyx instance.
+
+    This function is particularly useful when calling ONE methods in remote mode.
+
+    Parameters
+    ----------
+    ac : AlyxClient
+        An instance of the AlyxClient to modify.  If None, the a new object is instantiated
+
+    Returns
+    -------
+    AlyxClient
+        The instance of Alyx with cache disabled
+
+    Examples
+    --------
+    >>> from one.api import ONE
+    >>> with no_cache(ONE().alyx):
+    ...     eids = ONE().search(subject='foobar', query_type='remote')
+    """
+    ac = ac or AlyxClient()
+    cache_mode = ac.cache_mode
+    ac.cache_mode = None
+    try:
+        yield ac
+    finally:
+        ac.cache_mode = cache_mode
+
+
 class _PaginatedResponse(Mapping):
     """
     This class allows to emulate a list from a paginated response.
@@ -511,7 +543,7 @@ class AlyxClient():
             return
         else:
             try:
-                message = json.loads(r.text).get('detail')
+                message = json.loads(r.text)
             except json.decoder.JSONDecodeError:
                 message = r.text
             raise requests.HTTPError(r.status_code, rest_query, message, response=r)
